<?php


function FillAndInterpolateData($aXData,$aYData,$aSampleInterval=1) {

    // First do some sanity checks on the input data
    $nx = count($aXData);
    $ny = count($aYData);
    if( $nx != $ny )
        return array(false,-1);
        
    for( $i=0; $i < $nx; ++$i ) {
         if( count($aXData[$i]) != count($aYData[$i]) )
             return array(false,-2);
    }

    // Create the sorted union of all X-coordinates
    $unionx = array_union($aXData);
    $length = count($unionx);

    // We now have to make sure that the distance between all
    // X-coordinates is 1 unit of the sample interval. If not
    // we will have to insert suitable X-value
    $i=1;
    while( $i < $length ) {
	$missing = 0;
	$diff = $unionx[$i] - $unionx[$i-1];
	if( $diff != $aSampleInterval ) {
            // Sanity check to make sure sample interval is an even multiple
            // of the distance between the gven X-coordinates
            if( $diff % $aSampleInterval !== 0 ) {
              return array(false,-4);
            }

	    $missing = $diff / $aSampleInterval - 1;
	    $fill = array();
	    for( $j=0; $j < $missing; ++$j ) {
		$fill[$j] = $aSampleInterval*($j+1)+$unionx[$i-1];
	    }
	    $unionx = array_merge(
		array_slice($unionx,0,$i),$fill,array_slice($unionx,$i));
	}
	$i += $missing+1;
	$length += $missing;
    }

    if( $length != count($unionx) ) {
        // Internal error check
        return array(false,-3);
    }

    // Now loop through all the individual data sets
    // and find out which x-data is missing and hence
    // needs to be interpolated
    $n = count($aXData);

    for( $i=0; $i < $n; ++$i ) {
	$missing_values = array_diff($unionx, $aXData[$i]);

	// Now find the position of each missing X-coordinate
	// and use that position in the corresponding Y array
	// to insert an interpolated value
	$m = count($missing_values);
	foreach( $missing_values as $key => $val ) {
	    $idx = array_search($val,$unionx);

	    // Now split the Y-array at that position and insert
	    // a new sentinel value
	    if( $idx >= 0 ) {
		$aYData[$i] = array_merge(
		    array_slice($aYData[$i],0,$idx),
		    array(NULL),
		    array_slice($aYData[$i],$idx));
	    }
	}

	// The next step is to actually calculate an interplated value
	// for the Y-coordinates we don't have. As a special case any
	// beginning or ending non-defined coordinates are set to 0
	
	// Set all beginning NULL to 0
	for( $j=0; $j < $length; ++$j ) {
	    if( $aYData[$i][$j] !== NULL )
		break;
	    $aYData[$i][$j] = 0;
	}

	// Set all ending NULL to 0
	for( $j=$length-1; $j >= 0; --$j ) {
	    if( $aYData[$i][$j] !== NULL )
		break;
	    $aYData[$i][$j] = 0;
	}

	// Calculate the remaingin missing values as a linear
	// interpolation and keeping in mind that there might be
	// multiple missing values in a row.
	$j = 0;
	while($j < $length ) {
	    if( $aYData[$i][$j] === NULL ) {
		// How many unknown values in a row?
		$cnt = 1;
		while( $j+$cnt < $length && $aYData[$i][$j+$cnt]===NULL ) {
		    ++$cnt;
		}

		if( $cnt == 1 ) {
		    $aYData[$i][$j] = ($aYData[$i][$j-1]+$aYData[$i][$j+1])/2;
		}
		else {
		    $step = ($aYData[$i][$j+$cnt] - $aYData[$i][$j-1])/($cnt+1);
		    for( $k=1; $k <= $cnt; ++$k ) {
			$aYData[$i][$j+$k-1] = $step*$k+$aYData[$i][$j-1];
		    }
		}
	    }
	    ++$j;
	}
    }

    return array($unionx,$aYData);
}


//------------------------------------------------------------------------
// Helper function - that really should be a part of PHP standard library
//------------------------------------------------------------------------

// Create the sorted union of all numeric arrays given as argument
function array_union($a) {

    $n = count($a);
    $res = $a[0];
    for( $i=1; $i < $n; ++$i) {
	$res = _array2_union($res,$a[$i]);
    }
    sort($res);
    return $res;
};

// Return the union between two numeric arrays
function _array2_union($a,$b)
{
    if( $a == NULL ) return $b;
    if( $b == NULL ) return $a;

    // A standard "trick" to calculate the union of two arrays
    return array_merge(
	array_intersect($a,$b),
	array_diff($a, $b),
	array_diff($b, $a));
}

/*----------------------------------------------------------------------------
// Standard unit test pattern
//----------------------------------------------------------------------------
 */

/*
function _do_unit_test() {
    echo "Starting test ... ";
    $n = 10; // Number of random tests
    for( $i=0; $i < $n; ++$i ) {
	_unit_test();
	echo " $i,";flush();
    }

    echo "<br>Done.<br>";
}

// Approach the unit testing with a mote-carlo testing by creating
// various input data arrays with random gaps and then let the interpolation
// funciton patch them up.
function _unit_test() {

    $numsets = rand(1,5);
    $length = rand(1,27);
    $ydata = array();
    $samplerate = rand(1,97);

    // First generate a full set of data
    for( $i=0; $i < $numsets; ++$i ) {
	for( $j=0; $j < $length; ++$j ) {
	    $ydata[$i][$j] = rand(-100,100);
	    $xdata[$i][$j] = $j*$samplerate;
	}
    }

    // Then we randomly remove data pairs from each set
    for( $i=0; $i < $numsets; ++$i ) {
	// Determine home many pair we should remove
	$nremove = rand(0,$length-1);
	$size = $length;
	for( $j=0; $j < $nremove; ++$j ) {
	    $r = rand(0,$size);
	    $xdata[$i] = array_merge(array_slice($xdata[$i],0,$r),
				     array_slice($xdata[$i],$r+1));
	    $ydata[$i] = array_merge(array_slice($ydata[$i],0,$r),
				     array_slice($ydata[$i],$r+1));
	    --$size;
	}
    }    

    list($datax, $datay) = FillAndInterpolateData($xdata, $ydata);
    if( $datax === false ) {
	die( "Error: $datay" );
    }
}  

_do_unit_test();
*/

?>
